package app;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import common.exception.AppException;

public class ExcelUtil {
    static Logger logger = LoggerFactory.getLogger(ExcelUtil.class);
    /**
     *　Excelファイル読み込み 
     * @param sheetName	　シート名
     * @param cellInfoList　セル情報リスト
     * @param fpath　excelファイルパス
     * @throws AppException アプリケーション例外 
     */
    public void read(String fpath, String sheetName, List<CellInfo> cellInfoList ) throws AppException {

         logger.debug("app.ExcelUtil::read() start");

//        try {
            // Excelファイルへアクセス
//            Workbook excel = WorkbookFactory.create(new File(fpath));//実行Jar作成時、アボートしてしまう

            if (fpath.endsWith(".xls")) {
                // HSSFワークブック、ファイル読み込み
                readHSSF(fpath, sheetName, cellInfoList);
            }else {
                // XSSFワークブック、ファイル読み込み
                readXSSF(fpath, sheetName, cellInfoList);
            }
//        } 
//        catch (IndexOutOfBoundsException ex) {
//            throw new AppException("セルの値取得に失敗しました。", ex);
//        } 
        logger.debug("app.ExcelUtil::read() end");
        
    }
    /**
     * 旧EXCELファイル(*.xls)読み込み
     * @param fpath　excelファイルパス
     * @param sheetName	　シート名
     * @param cellInfoList　セル情報リスト
     * @throws AppException アプリケーション例外 
     */
    private void readHSSF(String fpath, String sheetName, List<CellInfo> cellInfoList ) throws AppException {
        // HSSFワークブックオブジェクト生成（この部分がXSSFファイルと異なる）
        POIFSFileSystem fs;
        try {
            fs = new POIFSFileSystem(new File(fpath));
            HSSFWorkbook excel = new HSSFWorkbook(fs.getRoot(), true);
            // シート名を取得
            Sheet sheet = excel.getSheet(sheetName);
            
            // 0行目のRowを取得
            CellInfo cellInfo = cellInfoList.get(0);
            Row row = sheet.getRow(cellInfo.getPosition().getRow());//インデックスcellInfo.getPosition().getRow() のRow取得
    
            for(int i=0; i < cellInfoList.size();i++) {
                // i番目のセルの値を取得
                Cell wkcell = row.getCell(cellInfoList.get(i).getPosition().getCell());//インデックスwkcellInfo.getPosition().getCell()　のCell取得
                // セルの値を文字列として取得
                String value = wkcell.getStringCellValue();
                logger.debug("app.ExcelUtil::read()  " + "セル（"+ String.valueOf(i)+ "）： " + value);
            }
            excel.close();
        } catch (IOException e) {
            throw new AppException(AppMain.msgMap.get("ERROR_FILE_IO"), e);
        }

    }
    /**
     * EXCELファイル(*.xlsx)読み込み
     * @param sheetName	　シート名
     * @param cellInfoList　セル情報リスト
     * @param fpath　excelファイルパス
     * @throws AppException アプリケーション例外 
     */
    private void readXSSF(String fpath, String sheetName, List<CellInfo> cellInfoList ) throws AppException {
        // XSSFワークブックオブジェクト生成（この部分がHSSFファイルと異なる）
        OPCPackage pkg;
        try {
            pkg = OPCPackage.open(fpath);
            XSSFWorkbook excel = new XSSFWorkbook(pkg);
            // シート名を取得
            XSSFSheet sheet = excel.getSheet(sheetName);
            
            // 0行目のRowを取得
            CellInfo cellInfo = cellInfoList.get(0);
            //XSSFRow
            XSSFRow row = sheet.getRow(cellInfo.getPosition().getRow());//インデックスcellInfo.getPosition().getRow() のRow取得
    
            for(int i=0; i < cellInfoList.size();i++) {
                // i番目のセルの値を取得
                XSSFCell wkcell = row.getCell(cellInfoList.get(i).getPosition().getCell());//インデックスwkcellInfo.getPosition().getCell()　のCell取得
                // セルの値を文字列として取得
                String value = wkcell.getStringCellValue();
                logger.debug("app.ExcelUtil::read()  " + "セル（"+ String.valueOf(i)+ "）： " + value);
            }
            excel.close();
        } catch (InvalidFormatException e) {
            throw new AppException(AppMain.msgMap.get("ERROR_EXCEL_BAD_FORMAT"), e);
        } catch (IOException e) {
            throw new AppException(AppMain.msgMap.get("ERROR_FILE_IO"), e);
        }
    }
    /**
     * Excelファイル保存
     * @param sheetName　シート名
     * @param cellInfoList　セル情報
     * @param fpath　Excelファイルパス
     * @throws AppException　アプリケーション例外 
     */
    public void write(String sheetName, List<CellInfo> cellInfoList,String fpath) throws AppException {
        logger.debug("app.ExcelUtil::write() start");
        
        try {
            // Excelファイルを作成
            Workbook outputWorkbook = new XSSFWorkbook();

            // シートを作成
            Sheet outputSheet = outputWorkbook.createSheet(sheetName);

            // 行を作成
            CellInfo cellInfo = cellInfoList.get(0);
            Row outputRow = outputSheet.createRow(cellInfo.getPosition().getRow());//インデックスcellInfo.getPosition().getRow()（0以上）のRow作成
            
            for(int i=0; i < cellInfoList.size(); i++) {
                Cell wkoutputCell = outputRow.createCell((cellInfoList.get(i)).getPosition().getCell());//インデックス (cellInfoList.get(i)).getPosition() のCell作成
                wkoutputCell.setCellValue((cellInfoList.get(i)).getValue());
            }

            // 文字色を設定
//            Font font = outputWorkbook.createFont();

//            font.setColor(org.apache.poi.ss.usermodel.IndexedColors.RED.getIndex());

//            CellStyle style1 = outputWorkbook.createCellStyle();
//        void setFillPattern(FillPatternType var1);
//            // セルの背景色を設定
//            style1.setFillPattern(org.apache.poi.ss.usermodel.FillPatternType.SOLID_FOREGROUND);
//            // セルのフォント設定
//            style1.setFont(font);
//
//            style1.setFillForegroundColor(IndexedColors.AQUA.getIndex());
//
//            outputCell_name.setCellStyle(style1);

            // 出力用のストリームを用意
            FileOutputStream out = new FileOutputStream(fpath);

            // ファイルへ出力
            outputWorkbook.write(out);

            outputWorkbook.close();
        } catch (FileNotFoundException ex) {
            throw new AppException(AppMain.msgMap.get("ERROR_FILE_NOT_FOUND"), ex);

        } catch (IOException ex) {
            throw new AppException(AppMain.msgMap.get("ERROR_FILE_IO"), ex);

        }
        logger.debug("app.ExcelUtil::write() end");

    }
}
